==> 0.4.0.tar.gz: lib/src/handler_pool.dart <==
// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

/// Handler pool base template
///
/// The handler pool manages handlers and condition mapping. Handlers have to be
/// added and triggered through this class.
class _HandlerPool<T extends Function> {
  /// Handler functions
  List<T> _handlers = new List<T>();

  /// Condition-less handlers
  Map<String, List<int>> _conditionLess = new Map<String, List<int>>();

  /// Pre-processed condition map for handlers
  Map<String, _ConditionMap> _conditionMap = new Map<String, _ConditionMap>();

  /// Insert new handler.
  int insert(String omnibusId, Map<String, dynamic> conditions, T handler) {
    // Check if handler exists.
    if (handler != null) {
      // Insert new handler.
      int index = _handlers.length;
      _handlers.add(handler);

      // Check if the condition map exists and contains items.
      if (conditions != null && conditions.isNotEmpty) {
        // Insert index in global condition map.
        _conditionMap.putIfAbsent(omnibusId, () => new _ConditionMap());
        _conditionMap[omnibusId].insertIndex(index, conditions);
      } else {
        // Insert handler in condition-less handler list.
        _conditionLess.putIfAbsent(omnibusId, () => new List<int>());
        _conditionLess[omnibusId].add(index);
      }

      return index;
    } else {
      throw new ArgumentError.notNull('handler');
    }
  }

  /// Retieve handler indices that match all given conditions.
  Set<int> getIndices(String omnibusId, Map<String, dynamic> conditions) {
    // Check if there are conditions.
    if (conditions != null && conditions.isNotEmpty) {
      // Check if a condition map exists.
      if (_conditionMap.containsKey(omnibusId)) {
        return _conditionMap[omnibusId].getIndices(conditions);
      } else {
        return new Set<int>();
      }
    } else if (_conditionLess.containsKey(omnibusId)) {
      // Return condition-less handlers.
      return _conditionLess[omnibusId].toSet();
    } else {
      return new Set<int>();
    }
  }
}

/// Function for handling event messages
typedef Future EventHandler(OmnibusEvent event);

/// _HandlerPool for EventHandlers
class _EventHandlerPool extends _HandlerPool<EventHandler> {
  /// Publish
  ///
  /// This method runs in the future and all handlers are executed
  /// asynchronously.
  Future<AsyncProcesses<Null>> publish(OmnibusEvent event) async {
    var indices = getIndices(event.omnibusId, event.getConditions());
    var futures = new Set<Future<Null>>();

    // Run event handlers.
    for (var i in indices) {
      futures.add(Function.apply(_handlers[i], [event]));
    }

    return new AsyncProcesses<Null>(futures);
  }
}

/// Function for handling request messages
typedef Future<OmnibusResponse> RequestHandler(OmnibusRequest request);

/// _HandlerPool for RequestHandlers
class _RequestHandlerPool extends _HandlerPool<RequestHandler> {
  /// Publish
  ///
  /// This method runs in the future and all handlers are executed
  /// asynchronously.
  Future<AsyncResponses> publish(OmnibusRequest request) async {
    var indices = getIndices(request.omnibusId, request.getConditions());
    var replies = new Set<Future<OmnibusResponse>>();

    // Collect replies.
    for (var i in indices) {
      replies.add(Function.apply(_handlers[i], [request]));
    }

    return new AsyncResponses(replies);
  }
}

==> 0.5.0.tar.gz: lib/src/handler_pool.dart <==
// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

/// Handler pool base template
///
/// The handler pool manages handlers and condition mapping. Handlers have to be
/// added and triggered through this class.
class _HandlerPool<T extends Function> {
  /// Handler functions
  List<T> _handlers = new List<T>();

  /// Universal handlers
  Set<int> _univeral = new Set<int>();

  /// Condition-less handlers
  Map<String, Set<int>> _conditionLess = new Map<String, Set<int>>();

  /// Pre-processed condition map for handlers
  Map<String, _HandlerIndex> _conditionMap = new Map<String, _HandlerIndex>();

  /// Insert new handler
  int insert(String omnibusId, Map<String, dynamic> conditions, T handler) {
    // Check if handler exists.
    if (handler != null) {
      // Insert new handler.
      int index = _handlers.length;
      _handlers.add(handler);

      // Check if the condition map exists and contains items.
      if (conditions != null && conditions.isNotEmpty) {
        // Insert index in global condition map.
        _conditionMap.putIfAbsent(omnibusId, () => new _HandlerIndex());
        _conditionMap[omnibusId].insertIndex(index, conditions);
      } else {
        // Insert handler in condition-less handler list.
        _conditionLess.putIfAbsent(omnibusId, () => new Set<int>());
        _conditionLess[omnibusId].add(index);
      }

      return index;
    } else {
      throw new ArgumentError.notNull('handler');
    }
  }

  /// Insert universal handler
  int insertUniversal(T handler) {
    // Check if handler exists.
    if (handler != null) {
      // Insert new handler.
      int index = _handlers.length;
      _handlers.add(handler);
      _univeral.add(index);
      return index;
    } else {
      throw new ArgumentError.notNull('handler');
    }
  }

  /// Retieve handler indices that match all given conditions.
  Set<int> getIndices(String omnibusId, Map<String, dynamic> messageData) {
    // Check if there are conditions.
    if (messageData != null && messageData.isNotEmpty) {
      // Check if a condition map exists.
      if (_conditionMap.containsKey(omnibusId)) {
        return _univeral
            .union(_conditionMap[omnibusId].findIndices(messageData));
      } else {
        return _univeral;
      }
    } else if (_conditionLess.containsKey(omnibusId)) {
      // Return condition-less handlers.
      return _univeral.union(_conditionLess[omnibusId]);
    } else {
      return _univeral;
    }
  }
}

/// Function for handling event messages
typedef Future EventHandler(Map<String, dynamic> event);

/// _HandlerPool for EventHandlers
class _EventHandlerPool extends _HandlerPool<EventHandler> {
  /// Publish
  ///
  /// This method runs in the future and all handlers are executed
  /// asynchronously.
  Future<AsyncProcesses<Null>> publish(OmnibusEvent event) async {
    var indices = getIndices(event.omnibusId, event.data);
    var futures = new Set<Future<Null>>();

    // Run event handlers.
    for (var i in indices) {
      futures.add(_handlers[i](event.data));
    }

    return new AsyncProcesses<Null>(futures);
  }
}

/// Function for handling request messages
typedef Future<OmnibusResponse> RequestHandler(Map<String, dynamic> message);

/// _HandlerPool for RequestHandlers
class _RequestHandlerPool extends _HandlerPool<RequestHandler> {
  /// Publish
  ///
  /// This method runs in the future and all handlers are executed
  /// asynchronously.
  Future<AsyncResponses> publish(OmnibusRequest request) async {
    var indices = getIndices(request.omnibusId, request.data);
    var replies = new Set<Future<OmnibusResponse>>();

    // Collect replies.
    for (var i in indices) {
      replies.add(_handlers[i](request.data));
    }

    return new AsyncResponses(replies);
  }
}
